@@ -73,8 +73,8 @@ actions = [
     (false, lambda s: (False, s)),
     (null, lambda s: (None, s)),
     (identifier, lambda s: (IdentifierToken(s), s)),
-    (longNumber, lambda s: (long(s), s)),
     (floatNumber, lambda s: (float(s), s)),
+    (longNumber, lambda s: (long(s), s)),
 ]
 def tokenise(s):
     tokens = []
@@ -90,7 +90,7 @@ def tokenise(s):
         if tok is not WhitespaceToken:
             tokens.append(tok)
         s = s[len(tokstr):]
-    
+
     return tokens
 
 def accept(want, tokens):
@@ -153,8 +153,8 @@ def parseObject(tokens):
         if not first:
             accept(',', tokens)
         first = False
-            
-        name, tokens = parseIdentifier(tokens)
+
+        name, tokens = parseString(tokens)
         accept(':', tokens)
         value, tokens = parseValue(tokens)
         o[name] = value
@@ -179,7 +179,7 @@ def _serialize(obj, w, seen):
             w('true')
         else:
             w('false')
-    elif isinstance(obj, (int, long)):
+    elif isinstance(obj, (int, long, float)):
         w(str(obj))
     elif isinstance(obj, unicode):
         w('"')
@@ -199,8 +199,7 @@ def _serialize(obj, w, seen):
     elif isinstance(obj, dict):
         w('{')
         for n, (k, v) in enumerate(obj.iteritems()):
-            assert isinstance(k, str)
-            w(k)
+            _serialize(k, w, seen)
             w(':')
             _serialize(v, w, seen)
             if n != len(obj) - 1:
